//@author: vux
//@help: standard constant shader
//@tags: color
//@credits: 

Texture2D texture2d <string uiname="Texture";>;

SamplerState g_samLinear
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

 
cbuffer cbPerDraw : register( b0 )
{
	float4x4 tVP : VIEWPROJECTION;
};


cbuffer cbPerObj : register( b1 )
{
	float4x4 tW : WORLD;
	float4 cAmb <bool color=true;String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };
	int viewportIndex;
};

struct vsInput
{
	float4 PosO : POSITION;
	float4 TexCd : TEXCOORD0;
};

struct gsInput
{
    float4 PosWVP: SV_POSITION;
    float4 TexCd: TEXCOORD0;
	uint vid : VIEWPORTINDEX;
};

struct gsOutput
{
    float4 PosWVP: SV_POSITION;
    float4 TexCd: TEXCOORD0;
	uint vid : SV_ViewportArrayIndex;
};

struct psInput
{
    float4 PosWVP: SV_POSITION;
    float4 TexCd: TEXCOORD0;
};


gsInput VS(vsInput input)
{
	gsInput output;
    output.PosWVP  = mul(input.PosO,mul(tW,tVP));
    output.TexCd = input.TexCd;
	output.vid = viewportIndex;
    return output;
}

[maxvertexcount(3)]
void GS( triangle gsInput input[3], inout TriangleStream<gsOutput> gsout )
{
	gsOutput output;
	
	[unroll]
	for (int i = 0; i < 3; i++)
	{
		output = input[i];
		gsout.Append(output);
	}
	gsout.RestartStrip();
}


float4 PS(psInput input): SV_Target
{
    float4 col = texture2d.Sample(g_samLinear,input.TexCd.xy) * cAmb;
    return col;
}

technique10 Constant
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetGeometryShader(CompileShader(gs_4_0,GS()));
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}




